/**
 * \file cella.h
 *    Cella és Labnyom objektum deklarációja a labirintus programhoz.
 * $Id: cella.h 499 2015-05-05 10:53:36Z szebi $
 */

#ifndef CELLA_H
#define CELLA_H

#include "obj.h"
#include <set>


/**
 * Lábnyom struktúra.
 * Az objektumok nyomának tárolására.
 * Minden cella egy halmazban tárolja a cellába belépett objektumok nyomait.
 * Objektum minden tagja publikus (struct), mivel csak egy privat halmazban fordulhat elő.
 * Kezelése viszont így egyszerűbb.
 */
struct Labnyom {
    const Obj *az;                              ///< azonosító
    int errol;                                  ///< erről jött ide

    /// Konstruktor, ez a default is.
    /// @param a - Nyomot hagyó objektum címe
    /// @param i - irány, amerről legutóbb jött
    Labnyom(const Obj *a = NULL, int i = 0)
        :az(a), errol(i) {}
};

/**
 * Lábnyom hasonlít.
 * Összehasonlító függvény a lábnyom set számára.
 * Nem az objektumokat kell hasonlítani, hanem az objektum "az" adattagját, ami
 * pointer. Csak az azonosság érdeles
 */
struct Has {
    bool operator()(const Labnyom& l1, const Labnyom& l2) {
        return l1.az < l2.az;
    }
};


/**
 * Cellaobjektum.
 * Ebből épül fel a labirintus.
 * A 4 szomszédra pointerek mutatnak.
 */
class Cella {
    int     x, y;                               ///< geometriai koordináták
    Cella   *sz[4];                             ///< szomszédok, ha NULL, akkor fal
    Obj     *p;                                 ///< cella tartalma. URES, ha nincs rajta senki
    std::set<Labnyom, Has> ny;                  ///< lábnyomok halmaza
    typedef std::set<Labnyom, Has>::iterator nyIter;
    Cella(const Cella&);                        // hogy ne lehessen használni
    Cella& operator=(const Cella&);             // hogy ne lehessen használni
public:
    /// Irányok: jobbra, le, balra, fel.
    /// (Kódolási trükk: a szemben levő irányok csak a 2-es bitben különböznek.
    /// Ezt később kihasználjuk!
    enum irany { J, L, B, F };

    /// Def. konstruktor
    Cella() { SetPos(0, 0); }                   // hogy ne maradjon inicializálatlan

    /// Cella geometriai pozíciójának beállítása
    /// @param i - sor koordináta
    /// @param j - oszlop koordináta
    void SetPos(int i, int j) {                 // szomszédok null értéket kapnak.
        x = i; y = j; p = NULL; sz[F] = sz[L] = sz[B] = sz[J] = NULL;}

    /// Szomszédság beállítása
    /// @param n  - irány (J, L, B, F)
    /// @param cp - adott irányba eső szomszéd cella pointere
    void SetSz(int n, Cella *cp) { sz[n] = cp; }// szomszéd beállítása

    /// Szomszédság lekérdezése
    /// @param n  - irány (J, L, B, F)
    /// @return   - adott irányba eső szomszéd cella pointere
    Cella *GetSz(int n) { return(sz[n]); }      // szomszéd lekérdezése

    /// Cellára objektumot tesz
    /// @param a  - Objektum pointere
    void SetObj(Obj *a) { p = a;}               // objektum beállítása

    /// Cellán levő objektum lekérdezése
    /// @return - Objektum pointere
    Obj *GetObj() { return(p); }                // objektum lekérdezése

    /// Cellán levő objektum nyomának beírása a cellalistába
    /// @param n - ebből az irányból lépett be a cellára
    void SetNyom(int n);                        // lábnyom beírása

    /// Objektum nyom irányának lekérdezése a cellalistából
    /// @param a - objektumra mutató pointer
    int GetNyom(const Obj *a);                  // lábnyom kiolvasása

    /// Objektum nyomának kitörlése a cellalistából
    void DelNyom() {ny.erase(Labnyom(GetObj()));} // lábnyom törlése
};

#endif
